use glam::{DVec3, DQuat};
use terrain::{
    icosphere::Icosphere,
    system::{StarSystem, Orbit},
    export::export_obj,
};

fn main()
{
    let ico = Icosphere::new();

    let systems = [
        5.,
        1.,
        0.5,
        1.5,
    ];

    let sys = StarSystem::singular(0, 10.)
    .append(
        StarSystem::singular(1, 1.)
        .append(
            StarSystem::singular(2, 0.5),
            Orbit::with(DQuat::IDENTITY, 2., 0.1, 5., 0.),
        ),
        Orbit::with(DQuat::IDENTITY, 20., 0.5, 10., 0.),
    )
    .append(
        StarSystem::singular(3, 1.),
        Orbit::with(DQuat::from_axis_angle(-DVec3::Z, f64::to_radians(30.)), 50., 0.1, 15., 0.),
    );

    let mut mesh = ico.to_mesh().scale(systems[0]);
    for frame in 0..20 {
        let time = frame as f64 * 0.5;
        for (id, position) in sys.resolve(time) {
            if id != 0 {
                mesh = mesh.merge(&ico.to_mesh().scale(systems[id]), position.as_vec3());
            }
        }
    }

    export_obj("system.obj", &mesh, 10.).ok();
}
