use std::io::Write;
use glam::Vec3;
use crate::mesh::Mesh;

pub fn export_obj(path:&str, mesh:&Mesh, scale:f32) -> Result<(),()>
{
    match std::fs::File::create(path) {
        Ok(mut file) => {
            let mut min = Vec3::ZERO;
            let mut max = Vec3::ZERO;

            for vertex in &mesh.vertices {
                let v = vertex.position * scale;
                if v.x < min.x { min.x = v.x } else if v.x > max.x { max.x = v.x };
                if v.z < min.z { min.z = v.z } else if v.z > max.z { max.z = v.z };

                file.write_all(format!("v {:.6} {:.6} {:.6} {:3} {:3} {:3}\n",
                    v.x, v.y, v.z,
                    vertex.color.x, vertex.color.y, vertex.color.z,
                ).as_bytes()).ok();
            }

            file.write_all("\n".as_bytes()).ok();

            let rx = max.x - min.x;
            let rz = max.z - min.z;

            for vertex in &mesh.vertices {
                let v = vertex.position * scale;
                let ux = ((v.x / rx) + 0.5).clamp(0., 1.);
                let uz = ((v.z / rz) + 0.5).clamp(0., 1.);

                file.write_all(format!("vt {:.6} {:.6}\n", ux, uz).as_bytes()).ok();
            }

            file.write_all("\n".as_bytes()).ok();

            for f in &mesh.faces {
                file.write_all(format!("f {}/{} {}/{} {}/{}\n",
                    f.x + 1, f.x + 1,
                    f.y + 1, f.y + 1,
                    f.z + 1, f.z + 1,
                ).as_bytes()).ok();
            }

            Ok(())
        }
        Err(_) => Err(()),
    }
}
