use glam::{Vec3, IVec3};

pub struct Vertex {
    pub position:Vec3,
    pub color:Vec3,
}
impl Vertex{
    pub fn new(position:Vec3, color:Vec3) -> Self
    {
        Self {
            position:position,
            color:color,
        }
    }
}

pub struct Mesh {
    pub vertices:Vec<Vertex>,
    pub faces:Vec<IVec3>,
}
impl Mesh {
    pub fn new() -> Self
    {
        Self {
            vertices:Vec::<Vertex>::new(),
            faces:Vec::<IVec3>::new(),
        }
    }

    pub fn merge(mut self, mesh:&Mesh, translation:Vec3) -> Self
    {
        let start = self.vertices.len() as i32;
        for vertex in &mesh.vertices {
            self.vertices.push(Vertex::new(vertex.position + translation, vertex.color));
        }

        for face in &mesh.faces {
            self.faces.push(IVec3::new(
                face.x + start,
                face.y + start,
                face.z + start,
            ));
        }
        self
    }

    pub fn scale(mut self, scale:f32) -> Self
    {
        for vertex in &mut self.vertices {
            vertex.position *= scale;
        }
        self
    }
}
