use glam::{Vec3, DVec3};

pub fn f64_sign(value:f64) -> f64
{
    ((value > 0.) as i32 - (value < 0.) as i32) as f64
}

pub fn barycentric_coordinates(positions:(DVec3, DVec3, DVec3), point:DVec3) -> DVec3
{
    let (a, b, c) = positions;

    let v0 = b - a;
    let v1 = c - a;
    let v2 = point - a;

    let d00 = v0.dot(v0);
    let d01 = v0.dot(v1);
    let d11 = v1.dot(v1);
    let d20 = v2.dot(v0);
    let d21 = v2.dot(v1);
    let denom = (d00 * d11) - (d01 * d01);
    let b0 = ((d11 * d20) - (d01 * d21)) / denom;
    let b1 = ((d00 * d21) - (d01 * d20)) / denom;
    let b2 = 1. - (b0 + b1);

    DVec3::new(b2, b0, b1)
}

pub fn barycentric_inside(positions:(DVec3, DVec3, DVec3), point:DVec3) -> bool
{
    let coords = barycentric_coordinates(positions, point);
    coords.x >= 0. && coords.x <= 1. && coords.y >= 0. && coords.y <= 1. && coords.z >= 0. && coords.z <= 1.
}

pub fn barycentric_value(positions:(DVec3, DVec3, DVec3), values:(f64, f64, f64), point:DVec3) -> f64
{
    let coords = barycentric_coordinates(positions, point);
    let (va, vb, vc) = values;
    (va * coords.x) + (vb * coords.y) + (vc * coords.z)
}

pub fn rgb(r:u8, g:u8, b:u8) -> Vec3
{
    Vec3::new(r as f32 / 255., g as f32 / 255., b as f32 / 255.)
}
